from .auth import *
from .actions import execute_js, redirect  # exported
from .socket import url_params  # exported
from .list_item_schema import ListItemSchema  # exported
from .page import Page  # exported
from .step import run_steps  # exported
from .generated.inputs import *
from .generated.outputs import *



from ..widget_base import Output
from urllib.parse import quote
from validators import url


class IframeOutput(Output):
    type = "iframe-output"

    def __init__(self, url_or_html: str, **kwargs):
        if url(url_or_html):
            self.url = url_or_html
        else:
            self.url = f"data:text/html,{quote(url_or_html)}"
        self.width = kwargs.get("width", "800")
        self.height = kwargs.get("height", "600")
        self.columns = kwargs.get("columns", 1)
        self.full_width = kwargs.get("full_width", False)

    def json(self, **kwargs):
        return {
            "type": self.type,
            "url": self.url,
            "width": self.width,
            "height": self.height,
            "columns": self.columns,
            "fullWidth": self.full_width,
        }



from typing import List


def get_suggestions(current_code, added_code_snippet) -> List[str]:
    import jedi

    jedi.settings.call_signatures_validity = 3.0

    complete_code = current_code + "\n" + added_code_snippet
    line = len(complete_code.split("\n"))
    column = len(added_code_snippet)
    script = jedi.Script(complete_code)
    completions = script.complete(line, column)
    result = []

    for c in completions:
        result.append(c.name)

    return result



class AuthResponse:
    """The response from the authentication process

    Attributes:
      email (str): The email address of the user
    """

    def __init__(self, email: str):
        self.email = email


def get_user() -> "AuthResponse":
    raise NotImplementedError("Must run inside server")



import unittest
from abstra_server.runtimes.dashes.program import PythonProgram
from .fixtures import init_dir, clear_dir
from abstra_server.api import API


class TestProgram(unittest.TestCase):
    def setUp(self) -> None:
        self.root = init_dir()

    def tearDown(self) -> None:
        clear_dir(self.root)

    def test_program_works(self):
        api = API(root=self.root)
        dash = api.create_dash()
        code = "a = 1"
        py = PythonProgram(dash, code)
        py.execute_initial_code()
        a = py.ev("a")
        self.assertEqual(a, 1)

        # testing set variable
        py.set_variable("a", 2)
        a = py.ev("a")
        self.assertEqual(a, 2)
